import logging
import inspect
import functools
import contextlib

from typing import Callable, Union, List, Optional

from ppa import appliance, ui, exceptions as sdk_exceptions
from ppa_api import client, exceptions

from .records import (
    Task,
    TaskData,
    DelayedTask,
    DelayedTaskData,
    TaskResult,
    TaskResultData,
    Image,
    ImageData,
    User,
    UserData,
    Group,
    GroupData,
)

logger = logging.getLogger(__name__)


ALL_RETURN_TYPES = Union[
    Task, List[Task], User, List[User], Group, List[Group], Image, List[Image], DelayedTask, List[DelayedTask], None
]


@contextlib.contextmanager
def get_client(address: Optional[str] = None, api_key: Optional[str] = None):
    # The address & api_key arguments should only be supplied when using dev-mode PPA.
    try:
        yield client.PPAClient(
            address if address is not None else appliance.address(),
            api_key=api_key if api_key is not None else None,
            verify=appliance.cert(),
            proxy=appliance.api_proxy(),
        )
    except exceptions.VersionError as e:
        # VersionError exceptions from this package already include
        # the method name, so we don't need to grab it ourselves.
        ui.output_error(str(e))
        raise e
    except Exception as e:
        # Get the name of the calling function. Usually the caller would be the first index, but
        # given plugin actions are decorated, the wrapped action is the second index. This may
        # change if some actions don't need decorating.
        caller_name = inspect.stack()[2].function
        ui.output_error(f"The '{caller_name}' action failed. {str(e)}")
        raise e


def create_records(record_type) -> Callable:
    try:
        record = {
            Task: TaskData,
            DelayedTask: DelayedTaskData,
            TaskResult: TaskResultData,
            Image: ImageData,
            User: UserData,
            Group: GroupData,
        }[record_type]
    except KeyError:
        raise KeyError(f"Only User, Task, DelayedTask, TaskResult, & Image records can be created. Received '{record_type}'.")

    def _create_wrapper(func: Callable) -> Callable:
        @functools.wraps(func)
        def _execute(*args, **kwargs) -> ALL_RETURN_TYPES:
            results = func(*args, **kwargs)
            logger.debug(results)
            if results is None:
                return None
            if not isinstance(results, list):
                return record(**results._asdict()).__dict__
            else:
                return [record(**result._asdict()).__dict__ for result in results]

        return _execute

    return _create_wrapper


# The min_ppa_version decorator in SDK doesn't display the error, this wrapped version does.
def min_ppa_version(minimum_version: str) -> Callable:
    def _wrapped_version_check(func):
        def _execute(*args, **kwargs):
            try:
                return appliance.min_ppa_version(minimum_version)(func)(*args, **kwargs)
            except sdk_exceptions.PPAVersionError as e:
                ui.output_error(str(e))
                raise e

        return _execute

    return _wrapped_version_check
