"""
Delayed Tasks

This module contains actions for creating delayed tasks in PPA & viewing delayed task history.

!!! danger "PPA Version"

    The actions in this module require PPA v2.8.x or newer.
"""
from typing import List, Union, Optional

from playbook_tools import tables

from ._api import get_client, create_records, min_ppa_version
from .records import DelayedTask


TABLE_HEADER = ["Task Name", "Description", "Created By", "Start Time", "Time Zone", "Pending"]
TABLE_FIELDS = ["image", "description", "username", "start_time", "timezone", "is_pending"]


@min_ppa_version("2.8.0")
@create_records(DelayedTask)
def get_all(address: Optional[str] = None, api_key: Optional[str] = None) -> List[DelayedTask]:
    """Get all delayed tasks visible to the current user.

!!! question "Minimum Plugin Version: 2.0.0"

??? info "Input"

    This action takes no inputs.

??? success "Output"

    A list of [DelayedTask](../inputs_outputs/#delayedtask) dictionaries.

??? example
    ```YAML linenums="1"
    - ppa_api.delayed_tasks.get_all:
      save: all_delayed_tasks
    ```
"""
    with get_client(address=address, api_key=api_key) as ppa:
        return ppa.delayed_tasks()


@min_ppa_version("2.8.0")
@create_records(DelayedTask)
def get_created_by_me(address: Optional[str] = None, api_key: Optional[str] = None) -> List[DelayedTask]:
    """Get all delayed tasks created by the current user.

!!! question "Minimum Plugin Version: 2.0.0"

??? info "Input"

    This action takes no inputs.

??? success "Output"

    A list of [DelayedTask](../inputs_outputs/#delayedtask) dictionaries.

??? example
    ```YAML linenums="1"
    - ppa_api.delayed_tasks.get_created_by_me:
      save: my_delayed_tasks
    ```
"""
    with get_client(address=address, api_key=api_key) as ppa:
        return ppa.tasks_delayed_by_me()


@min_ppa_version("2.8.0")
@create_records(DelayedTask)
def get_pending(address: Optional[str] = None, api_key: Optional[str] = None) -> List[DelayedTask]:
    """Get all _pending_ delayed tasks visible to the current user.

!!! question "Minimum Plugin Version: 2.0.0"

??? info "Input"

    This action takes no inputs.

??? success "Output"

    A list of [DelayedTask](../inputs_outputs/#delayedtask) dictionaries.

??? example
    ```YAML linenums="1"
    - ppa_api.delayed_tasks.get_pending:
      save: pending_delayed_tasks
    ```
"""
    with get_client(address=address, api_key=api_key) as ppa:
        return ppa.pending_delayed_tasks()


@min_ppa_version("2.8.0")
@create_records(DelayedTask)
def get_processed(address: Optional[str] = None, api_key: Optional[str] = None) -> List[DelayedTask]:
    """Get all _processed_ delayed tasks visible to the current user.

!!! question "Minimum Plugin Version: 2.0.0"

??? info "Input"

    This action takes no inputs.

??? success "Output"

    A list of [DelayedTask](../inputs_outputs/#delayedtask) dictionaries.

??? example
    ```YAML linenums="1"
    - ppa_api.delayed_tasks.get_processed:
      save: processed_delayed_tasks
    ```
"""
    with get_client(address=address, api_key=api_key) as ppa:
        return ppa.processed_delayed_tasks()


@min_ppa_version("2.8.0")
def display(text: str, delayed_tasks: Union[List[DelayedTask], DelayedTask]) -> None:
    """Display delayed tasks in a table.

The table will have the following columns:

- Task Name
- Description
- Created By
- Start Time
- Time Zone
- Pending

!!! question "Minimum Plugin Version: 3.0.0"

??? info "Input"

    - __text:__ the title of the table

    - __delayed_tasks:__ any number of [DelayedTasks](../inputs_outputs/#delayedtask) to display in the table

??? success "Output"

    Nothing is outputted by this action.

??? example
    ```YAML linenums="1"
    - ppa_api.delayed_tasks.get_all:
      save: delayed_tasks

    - ppa_api.delayed_tasks.display:
        text: All Delayed Tasks
      load:
        tasks: delayed_tasks
    ```
    """
    tables.output_table(text, header=TABLE_HEADER, extract=TABLE_FIELDS, records=delayed_tasks)
